@@ -5,6 +5,16 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+
+echo "WARNING: this script is deprecated and will be removed"
+echo "please use BUILD.py instead"
+for i in `seq 1 5`; do
+  echo -n "."
+  sleep 1
+done
+echo
+
+
 ARGS=$@
 DO_CLEAN=${DO_CLEAN:-1}
 DO_ZIP=${DO_ZIP:-0}
@@ -15,7 +25,7 @@ RUN_INSTALLER=${RUN_INSTALLER:-1}
 DO_MSI=${DO_MSI:-0}
 DO_SIGN=${DO_SIGN:-1}
 DO_TESTS=${DO_TESTS:-0}
-DO_SBOM=${DO_SBOM:1}
+DO_SBOM=${DO_SBOM:-1}
 
 # these are only enabled for "full" builds:
 DO_CUDA=${DO_CUDA:-$DO_FULL}
@@ -135,7 +145,7 @@ PYTHON_VERSION=`${PYTHON} --version | awk '{print $2}'`
 echo "Python version ${PYTHON_VERSION}"
 VERSION=`${PYTHON} -c "from xpra import __version__;import sys;sys.stdout.write(__version__)"`
 REVISION=`${PYTHON} -c "from xpra.src_info import REVISION;import sys;sys.stdout.write(str(int(REVISION)))"`
-ZERO_PADDED_VERSION=`${PYTHON} -c 'from xpra import __version__;print(".".join((__version__.split(".")+["0","0","0"])[:3]))'`".${NUMREVISION}"
+ZERO_PADDED_VERSION=`${PYTHON} -c 'from xpra import __version__;print(".".join((__version__.split(".")+["0","0","0"])[:3]))'`".${REVISION}"
 LOCAL_MODIFICATIONS=`${PYTHON} -c "from xpra.src_info import LOCAL_MODIFICATIONS;import sys;sys.stdout.write(str(LOCAL_MODIFICATIONS))"`
 FULL_VERSION=${VERSION}-r${REVISION}
 if [ "${LOCAL_MODIFICATIONS}" != "0" ]; then
@@ -350,7 +360,7 @@ find lib/ -name "*dll.a" -exec rm {} \;
 
 #only keep the actual loaders, not all the other crap cx_Freeze put there:
 mkdir lib/gdk-pixbuf-2.0/2.10.0/loaders.tmp
-mv lib/gdk-pixbuf-2.0/2.10.0/loaders/*pixbufloader*.dll lib/gdk-pixbuf-2.0/2.10.0/loaders.tmp/
+mv lib/gdk-pixbuf-2.0/2.10.0/loaders/libpixbufloader-*.dll lib/gdk-pixbuf-2.0/2.10.0/loaders.tmp/
 rm -fr lib/gdk-pixbuf-2.0/2.10.0/loaders
 mv lib/gdk-pixbuf-2.0/2.10.0/loaders.tmp lib/gdk-pixbuf-2.0/2.10.0/loaders
 if [ "${DO_FULL}" == "0" ]; then
@@ -501,21 +511,19 @@ rmdir xpra/* 2> /dev/null
 rm -fr zeroconf
 ZEROCONF_DIR=`$PYTHON -c "import zeroconf,os;print(os.path.dirname(zeroconf.__file__))"`
 cp -apr $ZEROCONF_DIR ./
+
 #leave ./lib
 popd > /dev/null
 #leave ./dist
 popd > /dev/null
-if [ "${DO_SBOM}" != "0" ]; then
-	if [ "${DO_FULL}" == "1" ]; then
-		./packaging/MSWindows/BUILD.py sbom --no-light
-		./packaging/MSWindows/BUILD.py export-sbom --no-light
-	else
-		./packaging/MSWindows/BUILD.py sbom
-		./packaging/MSWindows/BUILD.py export-sbom
-	fi
-fi
-pushd dist/lib > /dev/null
 
+if [ "${DO_SBOM}" != "0" ]; then
+  ./packaging/MSWindows/BUILD.py sbom
+  rm -f ./dist/lib/xpra/build_info*
+  cp xpra/build_info.py ./dist/lib/xpra/
+fi
+
+pushd dist/lib > /dev/null
 #zip up some modules:
 if [ "${ZIP_MODULES}" == "1" ]; then
 	#these modules contain native code or data files,
@@ -536,6 +544,7 @@ if [ "${ZIP_MODULES}" == "1" ]; then
 			http enum winreg copyreg _thread _dummythread builtins importlib \
 			logging queue urllib xml xmlrpc pyasn1_modules concurrent collections > /dev/null
 fi
+#leave ./dist/lib
 popd > /dev/null
 
 pushd dist > /dev/null
@@ -562,18 +571,20 @@ popd > /dev/null
 
 
 echo "* Adding manifest"
+sed "s/XPRA_ZERO_PADDED_VERSION/$ZERO_PADDED_VERSION/" packaging/MSWindows/exe.manifest > packaging/MSWindows/exe.manifest.tmp
 for exe in Bug_Report Xpra-Launcher Xpra Xpra_cmd; do
-  cp packaging/MSWindows/exe.manifest ./dist/${exe}.exe.manifest
+  cp packaging/MSWindows/exe.manifest.tmp ./dist/${exe}.exe.manifest
 done
 if [ "${DO_FULL}" == "1" ]; then
 	for exe in GTK_info NativeGUI_info Screenshot Xpra-Shadow; do
-	  cp packaging/MSWindows/exe.manifest ./dist/${exe}.exe.manifest
+	  cp packaging/MSWindows/exe.manifest.tmp ./dist/${exe}.exe.manifest
 	done
 fi
+rm packaging/MSWindows/exe.manifest.tmp
 
 
 echo "* Generating gdk pixbuf loaders.cache"
-gdk-pixbuf-query-loaders.exe dist/lib/gdk-pixbuf-2.0/2.10.0/loaders/* | sed 's+".*dist/+"+g' > dist/lib/gdk-pixbuf-2.0/2.10.0/loaders.cache
+gdk-pixbuf-query-loaders.exe "dist/lib/gdk-pixbuf-2.0/2.10.0/loaders/*" | sed 's+".*dist/+"+g' > dist/lib/gdk-pixbuf-2.0/2.10.0/loaders.cache
 echo "* Generating icons and theme cache"
 for itheme in `ls dist/share/icons/`; do
 	gtk-update-icon-cache.exe -t -i "dist/share/icons/${itheme}"
@@ -646,10 +657,10 @@ fi
 if [ "${DO_VERPATCH}" == "1" ]; then
 	for exe in `ls dist/*exe | grep -v Plink.exe`; do
 		tool_name=`echo $exe | sed 's+dist/++g;s+Xpra_++g;s+Xpra-++g;s+_+ +g;s+-+ +g;s+\.exe++g'`
-		verpatch $exe				//s desc "Xpra $tool_name"		//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+		verpatch $exe				//s desc "Xpra $tool_name"		//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2024" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 	done
 	verpatch dist/Xpra_cmd.exe		//s desc "Xpra command line"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
-	verpatch dist/Xpra.exe			//s desc "Xpra"					//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
+	verpatch dist/Xpra.exe			//s desc "Xpra"					//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2024" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 	if [ "${DO_FULL}" == "1" ]; then
 		verpatch dist/Xpra-Proxy.exe	//s desc "Xpra Proxy Server"	//va "${ZERO_PADDED_VERSION}" //s company "xpra.org" //s copyright "(c) xpra.org 2020" //s product "xpra" //pv "${ZERO_PADDED_VERSION}"
 	fi
