@@ -6,7 +6,7 @@
 
 
 def test_dbus(display, wid=1, flag=True):
-    import dbus     #@UnresolvedImport
+    import dbus
     bus = dbus.SessionBus()
     server = 'org.xpra.Server%i' % display
     service = bus.get_object(server, "/org/xpra/Server")
@@ -14,6 +14,7 @@ def test_dbus(display, wid=1, flag=True):
     print("calling %s.SetVideoRegionEnabled(%i, %s)" % (server, wid, flag))
     SetVideoRegionEnabled(wid, flag)
 
+
 def main():
     import sys
     if len(sys.argv)!=4:
@@ -21,9 +22,10 @@ def main():
         return
     DISPLAY = int(sys.argv[1])
     wid = int(sys.argv[2])
-    from xpra.scripts.config import parse_bool
-    flag = parse_bool("flag", sys.argv[3])
+    from xpra.scripts.config import str_to_bool
+    flag = str_to_bool(sys.argv[3])
     test_dbus(DISPLAY, wid, flag)
 
+
 if __name__ == "__main__":
     main()
